/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#include "benchmarks/bit_stream.h"
#include "base/clock.h"
#include "compression/wah_bit_vector.h"
#include "compression/wah_compression.h"
#include <iostream>
#include <fstream>
#include <cassert>

/**
 * Checks the decompressed stream against the original stream and
 * prints some error message, if they're not equal.
 * @param original_stream
 * @param decompressed_stream
 */
void validate_decompressed(aidbms::benchmarks::BitStream &original_stream, aidbms::benchmarks::BitStream &decompressed_stream)
{
    if(decompressed_stream.size() != original_stream.size()) {
        std::cerr
            << "Error while compression: Size is not equal!\n"
            << "  Original bits:     " << original_stream.size() << "\n"
            << "  Decompressed bits: " << decompressed_stream.size()
            << std::endl;
        return;
    }

    for(auto i = 0u; i < original_stream.size(); i++) {
        if(original_stream[i] != decompressed_stream[i]) {
            std::cerr
                << "Error while compression: Bits are not equal!\n"
                << "  Original at index " << i << ":     " << original_stream[i] << "\n"
                << "  Decompressed at index " << i << ": " << decompressed_stream[i]
                << std::endl;
        }
    }
}


/**
 * @brief print_compression_statistics
 * @param original_stream
 * @param compressed_vectors
 * @param decompressed_stream
 */
void print_compression_statistics(aidbms::benchmarks::BitStream &original_stream, std::vector<aidbms::compression::WAHBitVector> &compressed_vectors, aidbms::benchmarks::BitStream &decompressed_stream)
{
    std::cout << "Original:     " << original_stream.size() << " bits." << std::endl;
    std::cout << "Compressed:   " << compressed_vectors.size() * 32 << " bits." << std::endl;
    std::cout << "Decompressed: " << decompressed_stream.size() << " bits." << std::endl;
    std::cout << "Compression factor: " << decompressed_stream.size()/static_cast<float>(compressed_vectors.size() * 32) << std::endl << std::endl;
}


int main()
{
    aidbms::compression::WAHCompression wah_compression;


    /* Uncomment the following lines if you want to test your decompress-function */
    // auto compressed_bitstream = aidbms::benchmarks::BitStream::read_compressed_bitstream("workloads/compressed_bitstream.bit");
    // auto decompressed_bistream = wah_compression.decompress(compressed_bitstream);
    // wah_compression.output_file(decompressed_bistream, "workloads/compressed_bitstream.bit.decompressed");


    // Bitstreams to compress
    std::vector<std::string> filenames = {"workloads/bitstream_1.bit", "workloads/bitstream_2.bit"};

    // Set to true if you want to dump the compressed and decompressed bitstreams
    bool dump_output_to_file = true;


    for (auto filename : filenames) {
        aidbms::benchmarks::BitStream stream(filename.c_str());

        // Compress a bitstream
        auto compressed = wah_compression.compress(stream);

        // Decompress a compressed bitstream
        auto decompressed = wah_compression.decompress(compressed);

        // Check decompressed output and print some statistics
        validate_decompressed(stream, decompressed);
        print_compression_statistics(stream, compressed, decompressed);

        // Dump output to file
        if (dump_output_to_file){
            std::stringstream ss;
            ss << filename << ".compressed";
            wah_compression.output_file(compressed, ss.str().c_str());
            ss.str("");
            ss << filename << ".decompressed";
            wah_compression.output_file(decompressed, ss.str().c_str());
        }
    }
    return 0;
}
