/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2019 Databases and Information Systems Group TU Dortmund          *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include <vector>
#include <functional>
#include <regex>
#include <fstream>
#include <string>
#include <iostream>

namespace aidbms::benchmarks {
    /**
     * @brief The Workload class is a helper class to generate a workload for testing/benchmarks.
     */
    template<typename T> class Workload
    {
    public:
        using match_function = std::function<void(std::smatch&)>;

        inline typename std::vector<T>::const_iterator begin() const { return _items.begin(); }
        inline typename std::vector<T>::const_iterator end() const { return _items.end(); }
        inline typename std::vector<T>::size_type size() const { return _items.size(); }

        inline void add(T item) {
            _items.push_back(item);
        }

    protected:
        /**
         * @brief parse is a generic parse function that can be specialized via a regular expression
         * and a match functor.
         * @param file_name path to the file
         * @param expression expression to match per line
         * @param match_functor function to call for matched lines.
         */
        void parse(const char *file_name, std::regex &expression, match_function &match_functor) {
            std::smatch request_match;
            std::string line;
            std::ifstream file(file_name);

            while(std::getline(file, line)) {
                if(std::regex_match(line, request_match, expression)) {
                    match_functor(request_match);
                }
            }
        }

        std::vector<T> _items;
    };
}
